{ Kambi comments:

  This is JvDBGridExport modified by Kambi.
  One of my modifications (HandleException) needed to be done in
  implementation, I couldn't do this by simply inheriting new components.
  That's why I created this file copying it from jedi-vcl sources
  and modifying it.
  Some additional advantage of this is that it allows me to implement
  some things more cleanly:
  - I was able to move TJvDBGridHTMLExport.Header/Footer to private section
  - I was able to change DBGrid class from TDBGrid to TCustomDBGrid
    (To allow cleanly passing to it things like TKamCustomDBGrid;
    Well, actually, I needed to do similar dirty trick (declaring type
    TExposedDBGrid) to compile this, but now this dirty trick is hidden
    inside implementation of this class)
  However I needed to change class names. Now instead of "Jv" they are
  named "Kam".

  Other important Kambi changes:
  - Every field is exported (for every exporter) as it's Text property.
    This allows the greatest control for programmer how fields are displayed
    (he/she can use OnGetText events and various formatting properties),
    this is also how TCustomDBGrid displays fields' values.
  - Default export separator for CVS exporter is esSemiColon (instead of esTab),
    because this seems to be the only separator for M$ Excel to be able
    to read it.
    Although esComma is a standard separator for *comma*-separated values
    (CSV) format, but M$ Excel can't read it correctly.
  - added column headers to CSV export.
  - Removed brain-dead (IMHO) idea of HandleException, Silent and
    LastExceptionMessage properties. Now, if exporter fails to work, it just
    raises an exception, and you are freely to catch it and do whatever
    you like. Trying to catch it inside this unit caused some bugs -- 
    -- at first I corrected them, so they worked OK:

    I took the code of HandleException routine and pasted it to include
    file, then changed "raise ExceptObject at ExceptAddr" to simple
    "raise". Then changed in every place call to HandleException into
    inclusion of this new file. This fixed the problem with Delphi 7,
    because exception was now properly reraised.

    But still the code seem unclean.
    So I decided that the Right Thing is to simply change all this
    strange HandleException calls to simple "raise" to reraise the exception.
    Because this is usually the only sensible thing to do with the exception.
    In effect, in many cases the blocks "try ... except raise; end" were
    entirely eliminated.
  - XML exporter uses better attribute names, see ColumnXMLName
}

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvDBGridExport.pas, released on 2004-01-15

The Initial Developer of the Original Code is Lionel Renayud
Portions created by Lionel Renayud are Copyright (C) 2004 Lionel Renayud.
All Rights Reserved.

Contributor(s):

You may retrieve the latest version of this file at the Project JEDI's JVCL home page,
located at http://jvcl.sourceforge.net

Known Issues:
-----------------------------------------------------------------------------}
// $Id: JvDBGridExport.pas,v 1.26 2004/09/01 15:06:35 ahuser Exp $

unit KambiDBGridExport;

{$I jvcl.inc}
{$I windowsonly.inc}

interface

uses
  Windows, Classes, SysUtils, DB, DBGrids,
  JvComponent, JvSimpleXml, JvTypes;

type
  TExportDestination = (edFile, edClipboard);
  TExportSeparator = (esTab, esSemiColon, esComma, esSpace, esPipe);
  TWordOrientation = (woPortrait, woLandscape);

  EJvExportDBGridException = class(EJVCLException);
  TWordGridFormat = $10..$17;

  TOleServerClose = (scNever, scNewInstance, scAlways);

  TRecordColumn = record
    Visible: Boolean;
    Exportable: Boolean;
    ColumnName: string;
    Column: TColumn;
    Field: TField;
  end;

{ avoid Office TLB imports }
const
  wdDoNotSaveChanges = 0;

  wdTableFormatGrid1 = TWordGridFormat($10);
  wdTableFormatGrid2 = TWordGridFormat($11);
  wdTableFormatGrid3 = TWordGridFormat($12);
  wdTableFormatGrid4 = TWordGridFormat($13);
  wdTableFormatGrid5 = TWordGridFormat($14);
  wdTableFormatGrid6 = TWordGridFormat($15);
  wdTableFormatGrid7 = TWordGridFormat($16);
  wdTableFormatGrid8 = TWordGridFormat($17);

  xlPortrait = $01;
  xlLandscape = $02;

type
  TJvExportProgressEvent = procedure(Sender: TObject; Min, Max, Position: Cardinal;
    const AText: string; var AContinue: Boolean) of object;

  TKamCustomDBGridExport = class(TJvComponent)
  private
    FGrid: TCustomDBGrid; { Kambi*, /s/TDBGrid/TCustomDBGrid/ }
    FColumnCount: Integer;
    FRecordColumns: array of TRecordColumn;
    FCaption: string;
    FFileName: TFileName;
    FOnProgress: TJvExportProgressEvent;
    FOnException: TNotifyEvent;
    procedure CheckVisibleColumn;
  protected
    function ExportField(AField: TField): Boolean;
    function DoProgress(Min, Max, Position: Cardinal; const AText: string): Boolean; virtual;
    function DoExport: Boolean; virtual; abstract;
    procedure DoSave; virtual;
    procedure DoClose; virtual; abstract;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    function ExportGrid: Boolean;
  published
    // (p3) these should be published: all exporters must support them
    property Caption: string read FCaption write FCaption;
    { Kambi*, /s/TDBGrid/TCustomDBGrid/ }
    property Grid: TCustomDBGrid read FGrid write FGrid;
    property FileName: TFileName read FFileName write FFileName;
    property OnProgress: TJvExportProgressEvent read FOnProgress write FOnProgress;
    property OnException: TNotifyEvent read FOnException write FOnException;
  end;

  TKamCustomDBGridExportClass = class of TKamCustomDBGridExport;

  TKamDBGridWordExport = class(TKamCustomDBGridExport)
  private
    FWord: OleVariant;
    FVisible: Boolean;
    FOrientation: TWordOrientation;
    FWordFormat: TWordGridFormat;
    FClose: TOleServerClose;
    FRunningInstance: Boolean;
  protected
    procedure DoSave; override;
    function DoExport: Boolean; override;
    procedure DoClose; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property FileName;
    property Caption;
    property Grid;
    property OnProgress;
    property Close: TOleServerClose
      read FClose write FClose default scNewInstance;
    property WordFormat: TWordGridFormat
      read FWordFormat write FWordFormat default wdTableFormatGrid3;
    property Visible: Boolean read FVisible write FVisible default False;
    property Orientation: TWordOrientation
      read FOrientation write FOrientation default woPortrait;
  end;

  TKamDBGridExcelExport = class(TKamCustomDBGridExport)
  private
    FExcel: OleVariant;
    FVisible: Boolean;
    FAutoFit: Boolean;
    FOrientation: TWordOrientation;
    FClose: TOleServerClose;
    FRunningInstance: Boolean;
    function IndexFieldToExcel(Index: Integer): string;
  protected
    procedure DoSave; override;
    function DoExport: Boolean; override;
    procedure DoClose; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property FileName;
    property Caption;
    property Grid;
    property OnProgress;
    property Close: TOleServerClose read FClose write FClose default scNewInstance;
    property Visible: Boolean read FVisible write FVisible default False;
    property Orientation: TWordOrientation
      read FOrientation write FOrientation default woPortrait;
    property AutoFit: Boolean read FAutoFit write FAutoFit;
  end;

  TKamDBGridHTMLExport = class(TKamCustomDBGridExport)
  private
    FDocument: TStringList;
    FDocTitle: string;
    FHeader: TStringList;
    FFooter: TStringList;
    FIncludeColumnHeader: Boolean;
    function GetHeader: TStrings;
    function GetFooter: TStrings;
    procedure SetHeader(const Value: TStrings);
    procedure SetFooter(const Value: TStrings);
    { Kambi* moved this from published to private }
    property Header: TStrings read GetHeader write SetHeader;
    property Footer: TStrings read GetFooter write SetFooter;
  private
    { Kambi+ }
    FCharSet: string;
    FSomeComment: string;
    FDefaultExportInfo: boolean;
  protected
    procedure DoSave; override;
    function DoExport: Boolean; override;
    procedure DoClose; override;
    { Kambi- procedure SetDefaultData; }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property FileName;
    property Caption;
    property Grid;
    property OnProgress;
    property IncludeColumnHeader: Boolean
      read FIncludeColumnHeader write FIncludeColumnHeader default True;
    property DocTitle: string read FDocTitle write FDocTitle;

    { Kambi+ properties below }

    { If <> '' then we will append to Header line like
        <meta http-equiv="Content-Type" content="text/html; charset=#CHARSET">
      where #CHARSET is the value of this property.
      So CharSet should describe charset of DBGrid.Caption and text of
      all fields visible in DBGrid and value of DocTitle.

      Default is ''. }
    property CharSet: string read FCharSet write FCharSet;

    { This is some comment that will be placed in <head> of generated HTML.
      Don't use here '--' or '>' or '#TITLE' or '#STYLE' sequences.
      If not '', it will be enclosed inside '<!-- ' and ' -->'.

      Default is ''. }
    property SomeComment: string read FSomeComment write FSomeComment;

    { If true then an additional comment will be generated in exported
      file with some default exporting info, that is probably generally
      useful. It says what ProgramName generated this, and when (current Date
      and Time when DoExport method was called). }
    property DefaultExportInfo: boolean
      read FDefaultExportInfo write FDefaultExportInfo default true;
  end;

  TKamDBGridCSVExport = class(TKamCustomDBGridExport)
  private
    FDocument: TStringList;
    FDestination: TExportDestination;
    FExportSeparator: TExportSeparator;
    procedure SetExportSeparator(const Value: TExportSeparator);
    function SeparatorToString(ASeparator: TExportSeparator): string;
    procedure SetDestination(const Value: TExportDestination);
  protected
    function DoExport: Boolean; override;
    procedure DoSave; override;
    procedure DoClose; override;
  public
    Separator: string;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property FileName;
    property Caption;
    property Grid;
    property OnProgress;

    property Destination: TExportDestination
      read FDestination write SetDestination default edFile;
    property ExportSeparator: TExportSeparator
      read FExportSeparator write SetExportSeparator default esSemiColon;
  end;

  TKamDBGridXMLExport = class(TKamCustomDBGridExport)
  private
    FXML: TJvSimpleXML;
    FEncoding: string;
    function ClassNameNoT(AField: TField): string;
  protected
    function DoExport: Boolean; override;
    procedure DoSave; override;
    procedure DoClose; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { This is XML encoding of your data. Default is 'iso-8859-1'. }
    property Encoding: string read FEncoding write FEncoding;

    property FileName;
    property Caption;
    property Grid;
    property OnProgress;
  end;

function WordGridFormatIdentToInt(const Ident: string; var Value: Longint): Boolean;
function IntToWordGridFormatIdent(Value: Longint; var Ident: string): Boolean;
procedure GetWordGridFormatValues(Proc: TGetStrProc);

implementation

uses
  {$IFDEF UNITVERSIONING}
  JclUnitVersioning,
  {$ENDIF UNITVERSIONING}
  {$IFDEF HAS_UNIT_VARIANTS}
  Variants,
  {$ENDIF HAS_UNIT_VARIANTS}
  ComObj, Graphics, Clipbrd,
  JclRegistry,
  JvConsts, JvResources, KambiUtils;

{Kambi+}
type
  TExposedDBGrid = class(TCustomDBGrid)
  published
    property Columns;
    property Font;
  end;

//=== { TKamCustomDBGridExport } ==============================================

constructor TKamCustomDBGridExport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

function TKamCustomDBGridExport.DoProgress(Min, Max, Position: Cardinal;
  const AText: string): Boolean;
begin
  Result := True;
  if Assigned(FOnProgress) then
    FOnProgress(Self, Min, Max, Position, AText, Result);
end;

procedure TKamCustomDBGridExport.DoSave;
begin
  if FileExists(FileName) then
    DeleteFile(FileName);
end;

function TKamCustomDBGridExport.ExportField(AField: TField): Boolean;
begin
  Result := not (AField.DataType in [ftUnknown, ftBlob, ftGraphic,
    ftParadoxOle, ftDBaseOle, ftTypedBinary, ftCursor, ftADT,
    ftArray, ftReference, ftDataSet, ftOraBlob, ftOraClob, ftVariant,
    ftInterface, ftIDispatch, ftGuid]);
end;

procedure TKamCustomDBGridExport.CheckVisibleColumn;
var
  I: Integer;
begin
  FColumnCount := TExposedDBGrid(Grid).Columns.Count;
  SetLength(FRecordColumns, FColumnCount);
  for I := 0 to FColumnCount - 1 do
  begin
    FRecordColumns[I].Column := TExposedDBGrid(Grid).Columns[I];
    FRecordColumns[I].Visible := TExposedDBGrid(Grid).Columns[I].Visible;
    FRecordColumns[I].ColumnName := TExposedDBGrid(Grid).Columns[I].Title.Caption;
    FRecordColumns[I].Field := TExposedDBGrid(Grid).Columns[I].Field;
    if FRecordColumns[I].Visible and (FRecordColumns[I].Field <> nil) then
      FRecordColumns[I].Exportable := ExportField(FRecordColumns[I].Field)
    else
      FRecordColumns[I].Exportable := False;
  end;
end;

function TKamCustomDBGridExport.ExportGrid: Boolean;
begin
  if not Assigned(Grid) then
    raise EJvExportDBGridException.CreateRes(@RsEGridIsUnassigned);
  if not Assigned(Grid.DataSource) or not Assigned(Grid.DataSource.DataSet) then
    raise EJvExportDBGridException.CreateRes(@RsEDataSetDataSourceIsUnassigned);
//  if FileName = '' then
//    raise EJvExportDBGridException.Create(RsFilenameEmpty);
  CheckVisibleColumn;

  { Kambi*: Note that DoClose will be called even in case we exit from
    DoExport or DoSave with exception.

    This is required by Excel and Word exporter
    Proof: init some export to Word, cancel it by calling exception from
    OnProgress (this was probably not the scenario that original author
    of this code imagined -- he imagined that you will cancel by setting
    CanContinue to false in OnProgress and that you will use Silent = true
    and then HandleException and LastExceptionMessage work). Then (wthout
    destroying TKamCustomDBGridExport instance) open Word, close it,
    then get back to our program and close it, freeing
    TKamCustomDBGridExport instance. Effect ?
    TKamCustomDBGridExport instance tries errorneously to close Word
    application. Which it should not do, because
    1. user explicitly opened Word, so our program should not control Word
       anymore
    2. Word is already closed, so closing it will fail with exception... }

  try
   Result := DoExport;
   if Result then
     DoSave;
  finally DoClose end;
end;

procedure TKamCustomDBGridExport.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = Grid) then
    Grid := nil;
end;

//=== { TKamDBGridWordExport } ================================================

constructor TKamDBGridWordExport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Caption := RsExportWord;
  FWord := Unassigned;
  FVisible := False;
  FOrientation := woPortrait;
  FWordFormat := wdTableFormatGrid3;
  FClose := scNewInstance;
end;

destructor TKamDBGridWordExport.Destroy;
begin
  DoClose;
  inherited Destroy;
end;

function TKamDBGridWordExport.DoExport: Boolean;
const
  cWordApplication = 'Word.Application';
var
  I, J, K: Integer;
  lTable: OleVariant;
  ARecNo, lRecCount: Integer;
  lColVisible: Integer;
  lRowCount: Integer;
  lBookmark: TBookmark;
begin
  try
    Result := True;
    FRunningInstance := True;
    try
      // get running instance
      FWord := GetActiveOleObject(cWordApplication);
    except
      FRunningInstance := False;
      try
        // create new
        FWord := CreateOleObject(cWordApplication);
      except
        FWord := Unassigned;
        raise;
  //      raise EJvExportDBGridException.Create(RsNoWordApplication);
      end;
    end;

    if VarIsEmpty(FWord) then
      Exit;

    FWord.Visible := FVisible;
    FWord.Documents.Add;

    lColVisible := 0;
    for I := 1 to FColumnCount do
      if TExposedDBGrid(Grid).Columns[I - 1].Visible then
        Inc(lColVisible);

    lRowCount := Grid.DataSource.DataSet.RecordCount;
    FWord.ActiveDocument.Range.Font.Name := TExposedDBGrid(Grid).Font.Name;
    FWord.ActiveDocument.Range.Font.Size := TExposedDBGrid(Grid).Font.Size;
    if Orientation = woPortrait then
      FWord.ActiveDocument.PageSetup.Orientation := 0
    else
      FWord.ActiveDocument.PageSetup.Orientation := 1;
    lTable := FWord.ActiveDocument.Tables.Add(FWord.ActiveDocument.Range, lRowCount + 1, lColVisible);
    FWord.ActiveDocument.Range.InsertAfter('Date ' + DateTimeToStr(Now));
    // (rom) This is correct Delphi. See "positional parameters" in the Delphi help.
    lTable.AutoFormat(Format := WordFormat); // FormatNum, 1, 1, 1, 1, 1, 0, 0, 0, 1

    K := 1;
    for I := 0 to FColumnCount - 1 do
      if FRecordColumns[I].Visible then
      begin
        lTable.Cell(1, K).Range.InsertAfter(FRecordColumns[I].ColumnName);
        Inc(K);
      end;

    J := 2;
    with Grid.DataSource.DataSet do
    begin
      lRecCount := RecordCount;
      ARecNo := 0;
      DoProgress(0, lRecCount, ARecNo, Caption);
      DisableControls;
      lBookmark := GetBookmark;
      First;
      try
        while not Eof do
        begin
          K := 1;
          for I := 0 to FColumnCount - 1 do
          begin
            if FRecordColumns[I].Exportable and not FRecordColumns[I].Field.IsNull then
            try
              lTable.Cell(J, K).Range.InsertAfter(
                {Kambi*string(FRecordColumns[I].Field.Value)}FRecordColumns[I].Field.Text);
            except
              Result := False;
              raise;
              // Remember problem but continue
            end;
            if FRecordColumns[I].Visible then
              Inc(K);
          end;
          Next;
          Inc(J);
          Inc(ARecNo);
          if not DoProgress(0, lRecCount, ARecNo, Caption) then
            Last;
        end;
        DoProgress(0, lRecCount, lRecCount, Caption);
      finally
        if BookmarkValid(lBookmark) then
          GotoBookmark(lBookmark);
        if lBookmark <> nil then
          FreeBookmark(lBookmark);
        EnableControls;
      end;
    end;
    lTable.UpdateAutoFormat;
  except
    { Kambi+, better message (instead of mysterious "Nieprawidlowy ciag klasy") }
    on E: EOleSysError do
    begin
     E.Message := 'Wystpi bd przy prbie eksportu do Microsoft Worda. '+
       'Upewnij si e Microsoft Word jest prawidowo zainstalowany. '+
       'Szczegowy komunikat bdu: ' + E.Message;
     raise;
    end;
  end;
end;

procedure TKamDBGridWordExport.DoSave;
var
  lName: OleVariant;
begin
  inherited DoSave;
  if VarIsEmpty(FWord) then
    Exit;
  lName := OleVariant(FileName);
  FWord.ActiveDocument.SaveAs(lName);
end;

procedure TKamDBGridWordExport.DoClose;
begin
  if not VarIsEmpty(FWord) and (FClose <> scNever) then
  begin
    if (FClose = scAlways) or not FRunningInstance then
    begin
      FWord.ActiveDocument.Close(wdDoNotSaveChanges, EmptyParam, EmptyParam);
      FWord.Quit;
    end;
    FWord := Unassigned;
  end;
end;

//=== { TKamDBGridExcelExport } ===============================================

constructor TKamDBGridExcelExport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Caption := RsExportExcel;
  FExcel := Unassigned;
  FVisible := False;
  FOrientation := woPortrait;
  FClose := scNewInstance;
end;

destructor TKamDBGridExcelExport.Destroy;
begin
  DoClose;
  inherited Destroy;
end;

function TKamDBGridExcelExport.IndexFieldToExcel(Index: Integer): string;
begin
  // Max column : ZZ => Index = 702
  if Index > 26 then
    Result := Chr(64 + ((Index - 1) div 26)) + Chr(65 + ((Index - 1) mod 26))
  else
    Result := Chr(64 + Index);
end;

function TKamDBGridExcelExport.DoExport: Boolean;
const
  cExcelApplication = 'Excel.Application';
var
  I, J, K: Integer;
  lTable: OleVariant;
  lCell: OleVariant;
  ARecNo, lRecCount: Integer;
  lBookmark: TBookmark;
begin
  try
    Result := True;
    FRunningInstance := True;
    try
      // get running instance
      FExcel := GetActiveOleObject(cExcelApplication);
    except
      FRunningInstance := False;
      try
        // create new instance
        FExcel := CreateOleObject(cExcelApplication);
      except
        FExcel := Unassigned;
        raise;
      end;
    end;

    if VarIsEmpty(FExcel) then
      Exit;
      
    FExcel.WorkBooks.Add;
    FExcel.Visible := Visible;

    lTable := FExcel.ActiveWorkbook.ActiveSheet;

    if Orientation = woPortrait then
      lTable.PageSetup.Orientation := xlPortrait
    else
      lTable.PageSetup.Orientation := xlLandscape;

    K := 1;
    for I := 0 to FColumnCount - 1 do
      if FRecordColumns[I].Visible then
      begin
        lCell := lTable.Range[IndexFieldToExcel(K) + '1'];
        lCell.Value := FRecordColumns[I].ColumnName;
        Inc(K);
      end;

    J := 1;
    with Grid.DataSource.DataSet do
    begin
      ARecNo := 0;
      lRecCount := RecordCount;
      DoProgress(0, lRecCount, ARecNo, Caption);
      DisableControls;
      lBookmark := GetBookmark;
      First;
      try
        while not Eof do
        begin
          Inc(J);
          K := 1;
          for I := 0 to FColumnCount - 1 do
          begin
            if FRecordColumns[I].Exportable then
            begin
              lCell := lTable.Range[IndexFieldToExcel(K) + IntToStr(J)];
              try
                // Do not cast with string !
                lCell.Value := {Kambi*FRecordColumns[I].Field.Value}FRecordColumns[I].Field.Text;
              except
                Result := False;
                raise;
              end;
            end;
            if FRecordColumns[I].Visible then
              Inc(K);
          end;
          Next;
          Inc(ARecNo);
          if not DoProgress(0, lRecCount, ARecNo, Caption) then
            Last;
        end;
        if AutoFit then
          try
            lTable.Columns.AutoFit; // NEW! Autofit!
          except
             {$IFDEF DEBUGINFO_ON}
             on E: Exception do
               OutputDebugString(PChar('lTable.Columns.AutoFit failed. ' + E.Message));
             {$ENDIF DEBUGINFO_ON}
          end;
        DoProgress(0, lRecCount, lRecCount, Caption);
      finally
        if BookmarkValid(lBookmark) then
          GotoBookmark(lBookmark);
        if lBookmark <> nil then
          FreeBookmark(lBookmark);
        EnableControls;
      end;
    end;
  except
    { Kambi+, better message (instead of mysterious "Nieprawidlowy ciag klasy") }
    on E: EOleSysError do
    begin
     E.Message := 'Wystpi bd przy prbie eksportu do Microsoft Excela. '+
       'Upewnij si e Microsoft Excel jest prawidowo zainstalowany. '+
       'Szczegowy komunikat bdu: ' + E.Message;
     raise;
    end;
  end;
end;

procedure TKamDBGridExcelExport.DoSave;
var
  lName: OleVariant;
begin
  inherited DoSave;
  if not VarIsEmpty(FExcel) then
  begin
    lName := OleVariant(FileName);
    FExcel.ActiveWorkbook.SaveAs(lName);
  end;
end;

procedure TKamDBGridExcelExport.DoClose;
begin
  if not VarIsEmpty(FExcel) and (FClose = scNever) then
  begin
    FExcel.Visible := True;
    Exit;
  end;

  if not VarIsEmpty(FExcel) and (FClose <> scNever) then
  begin
    FExcel.ActiveWorkbook.Saved := True; // Avoid Excel's save prompt
    if (Close = scAlways) or not FRunningInstance then
    begin
      FExcel.ActiveWorkbook.Close;
      FExcel.Quit;
    end;
    FExcel := Unassigned;
  end;
end;

//=== { TKamDBGridHTMLExport } ================================================

constructor TKamDBGridHTMLExport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDocument := TStringList.Create;
  Caption := RsExportHTML;
  FDocTitle := RsHTMLExportDocTitle;
  FHeader := TStringList.Create;
  FFooter := TStringList.Create;
  FIncludeColumnHeader := True;
  { Kambi- SetDefaultData; }
  { Kambi+ }
  FDefaultExportInfo := true;
end;

destructor TKamDBGridHTMLExport.Destroy;
begin
  FFooter.Free;
  FHeader.Free;
  FDocument.Free;
  inherited Destroy;
end;

{ Kambi-
procedure TKamDBGridHTMLExport.SetDefaultData;
begin
  Header.Add('<html><head><title><#TITLE></title>');
  Header.Add('<style type=text/css>');
  Header.Add('#STYLE');
  Header.Add('</style>');
  Header.Add('</head><body>');

  Footer.Add('</body></html>');
end;
}

function TKamDBGridHTMLExport.GetFooter: TStrings;
begin
  Result := FFooter;
end;

procedure TKamDBGridHTMLExport.SetFooter(const Value: TStrings);
begin
  FFooter.Assign(Value);
end;

function TKamDBGridHTMLExport.GetHeader: TStrings;
begin
  Result := FHeader;
end;

procedure TKamDBGridHTMLExport.SetHeader(const Value: TStrings);
begin
  FHeader.Assign(Value);
end;

procedure TKamDBGridHTMLExport.DoClose;
begin
 // do nothing
end;

function TKamDBGridHTMLExport.DoExport: Boolean;

  { Kambi+ }
  procedure SetHeaderFooter;
  begin
   { calculate Header now }
   Header.Clear;
   Header.Add('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"');
   Header.Add('  "http://www.w3.org/TR/html4/loose.dtd">');
   Header.Add('<html>');
   Header.Add('<head>');
   Header.Add('  <title><#TITLE></title>');

   if DefaultExportInfo then
    Header.Add('  <!-- Generated by ' +ProgramName+ ' on '
      +DateTimeToAtStr(Now)+ ' -->');

   if SomeComment <> '' then
    Header.Add('  <!-- ' +SomeComment+ ' -->');

   if CharSet <> '' then
    Header.Add('  <meta http-equiv="Content-Type" content="text/html; charset='
      +CharSet+ '">');

   Header.Add('');
   Header.Add('  <style type=text/css>');
   Header.Add('    TH {padding: 0.1em}');
   Header.Add('    TD {padding: 0.1em}');
   Header.Add('  </style>');
   Header.Add('');

   Header.Add('  <style type=text/css>');
   Header.Add('#STYLE');
   Header.Add('  </style>');
   Header.Add('</head>');
   Header.Add('<body>');

   { calculate Footer now }
   Footer.Clear;
   Footer.Add('</body></html>');
  end;

var
  I: Integer;
  ARecNo, lRecCount: Integer;
  lBookmark: TBookmark;
  lString, lText, lHeader, lStyle: string;

  function AlignmentToHTML(AAlign: TAlignment): string;
  begin
    case AAlign of
      taLeftJustify:
        Result := 'left';
      taRightJustify:
        Result := 'right';
      taCenter:
        Result := 'center';
    end;
  end;

  function ColorToHTML(AColor: TColor): string;
  var
    r, g, b: byte;
  begin
    AColor := ColorToRGB(AColor);
    r := GetRValue(AColor);
    g := GetGValue(AColor);
    b := GetBValue(AColor);
    Result := Format('%.2x%.2x%.2x', [r, g, b]);
  end;

  function FontSubstitute(const Name: string): string;
  const
    cFontKey: array [Boolean] of PChar =
     ('SOFTWARE\Microsoft\Windows\CurrentVersion\FontSubstitutes',
      'SOFTWARE\Microsoft\Windows NT\CurrentVersion\FontSubstitutes');
  begin
    Result := RegReadStringDef(HKEY_LOCAL_MACHINE,
      cFontKey[Win32Platform = VER_PLATFORM_WIN32_NT], Name, Name);
  end;

  function FontSizeToHTML(PtSize: Integer): Integer;
  begin
    case Abs(PtSize) of
      0..8:
        Result := 1;
      9..10:
        Result := 2;
      11..12:
        Result := 3;
      13..17:
        Result := 4;
      18..23:
        Result := 5;
      24..35:
        Result := 6;
    else
      Result := 7;
    end;
  end;

  function FontToHTML(AFont: TFont; EncloseText: string): string;
  begin
    if fsBold in AFont.Style then
      EncloseText := '<b>' + EncloseText + '</b>';
    if fsItalic in AFont.Style then
      EncloseText := '<i>' + EncloseText + '</i>';
    if fsUnderline in AFont.Style then
      EncloseText := '<u>' + EncloseText + '</u>';
    if fsStrikeout in AFont.Style then
      EncloseText := '<s>' + EncloseText + '</s>';
    Result := Format('<font face="%s" color="#%s" size="%d">%s</font>',
      [FontSubstitute(AFont.Name), ColorToHTML(AFont.Color), FontSizeToHTML(AFont.Size), EncloseText]);
  end;

  function FontStyleToHTML(AFont: TFont): string;
  begin
    Result := '';
    if fsBold in AFont.Style then
      Result := 'FONT-WEIGHT: bold; ';
    if fsItalic in AFont.Style then
      Result := Result + 'FONT-STYLE: italic; ';
    if fsUnderline in AFont.Style then
      if fsStrikeout in AFont.Style then
        Result := Result + 'TEXT-DECORATION: underline line-through; '
      else
        Result := Result + 'TEXT-DECORATION: underline; '
    else
    if fsStrikeout in AFont.Style then
      Result := Result + 'TEXT-DECORATION: line-through; ';
  end;

begin
  SetHeaderFooter; {Kambi+}

  FDocument.Clear;

  Result := True;

  // Create Style like :
  //.Column0 {FONT-FAMILY: Arial; FONT-SIZE: 12px; FONT-WEIGHT: bold; FONT-STYLE: italic
  //      TEXT-ALIGN: right; COLOR: #FFFFFF; BACKGROUND: #9924A7}

  lStyle := '';
  lString := '<tr>';
  for I := 0 to FColumnCount - 1 do
    if FRecordColumns[I].Visible then
      with FRecordColumns[I].Column do
      begin
        lString := lString + Format('<th bgcolor="#%s" align="%s">%s</th>',
          [ColorToHTML(Title.Color), AlignmentToHTML(Alignment), FontToHTML(Title.Font, Title.Caption)]);
        lStyle := lStyle +
          Format('.Column%d {FONT-FAMILY: %s; FONT-SIZE: %dpt; %s TEXT-ALIGN: %s; COLOR: #%s; BACKGROUND: #%s;}'#13#10,
          [I, FontSubstitute(Font.Name), Font.Size, FontStyleToHTML(Font),
          AlignmentToHTML(Alignment), ColorToHTML(Font.Color), ColorToHTML(Color)]);
      end;
  lString := lString + '</tr>';
  lHeader := StringReplace(Header.Text, '<#TITLE>', DocTitle, [rfReplaceAll, rfIgnoreCase]);
  lHeader := StringReplace(lHeader, '#STYLE', lStyle, [rfReplaceAll, rfIgnoreCase]);

  FDocument.Add(lHeader);
  FDocument.Add('<table width="90%" border="1" cellspacing="0" cellpadding="0">');
  if IncludeColumnHeader then
    FDocument.Add(lString);

  with Grid.DataSource.DataSet do
  begin
    ARecNo := 0;
    lRecCount := RecordCount;
    DoProgress(0, lRecCount, ARecNo, Caption);
    DisableControls;
    lBookmark := GetBookmark;
    First;
    try
      while not Eof do
      begin
        lString := '<tr>';
        for I := 0 to FColumnCount - 1 do
          with FRecordColumns[I] do
            if Visible then
            begin
              if Exportable and not Field.IsNull then
              try
                lText := Field.{Kambi*AsString}Text;
                if lText = '' then
                  lText := '&nbsp;';
              except
                Result := False;
                raise;
              end
              else
                lText := '&nbsp;';

              lString := lString + Format('<td class="column%d">%s</td>',
                [I, lText]);
            end;
        lString := lString + '</tr>';
        FDocument.Add(lString);
        Next;
        {Kambi+, someone forgot about this}Inc(ARecNo);
        if not DoProgress(0, lRecCount, ARecNo, Caption) then
          Last;
      end;
      FDocument.Add('</table>');
      FDocument.AddStrings(Footer);
      DoProgress(0, lRecCount, lRecCount, Caption);
    finally
      if BookmarkValid(lBookmark) then
        GotoBookmark(lBookmark);
      if lBookmark <> nil then
        FreeBookmark(lBookmark);
      EnableControls;
    end;
  end;
end;

procedure TKamDBGridHTMLExport.DoSave;
begin
  inherited DoSave;
  FDocument.SaveToFile(FileName);
end;

//=== { TKamDBGridCSVExport } =================================================

constructor TKamDBGridCSVExport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDocument := TStringList.Create;
  FDestination := edFile;
  ExportSeparator := esSemiColon;
  Caption := RsExportFile;
end;

destructor TKamDBGridCSVExport.Destroy;
begin
  FDocument.Free;
  inherited Destroy;
end;

function TKamDBGridCSVExport.SeparatorToString(ASeparator: TExportSeparator): string;
begin
  case ASeparator of
    esTab:
      Result := Tab;
    esSemiColon:
      Result := ';';
    esComma:
      Result := ',';
    esSpace:
      Result := ' ';
    esPipe:
      Result := '|';
  end;
end;

procedure TKamDBGridCSVExport.SetExportSeparator(const Value: TExportSeparator);
begin
  FExportSeparator := Value;
  Separator := SeparatorToString(FExportSeparator);
end;

procedure TKamDBGridCSVExport.SetDestination(const Value: TExportDestination);
begin
  FDestination := Value;
  if FDestination = edFile then
    Caption := RsExportFile
  else
    Caption := RsExportClipboard;
end;

function TKamDBGridCSVExport.DoExport: Boolean;

  function MaybeQuote(const S: string): string;
  begin
   if CharsPos(AllChars - ['0'..'9', '.'], S) <> 0 then
    { Rather aggressive adding '="..."' is needed so that Excel will not
      fuck up columns (e.g. '12 A' will be changed to '12:00 am') }
    Result := '="' + S + '"' else
    Result := S;
  end;

var
  I: Integer;
  ARecNo, lRecCount: Integer;
  lBookmark: TBookmark;
  lString, lField: string;
begin
  FDocument.Clear;
  Result := True;

  with Grid.DataSource.DataSet do
  begin
    ARecNo := 0;
    lRecCount := RecordCount;
    DoProgress(0, lRecCount, ARecNo, Caption);
    DisableControls;
    lBookmark := GetBookmark;
    First;
    try
      {Kambi+}
      lString := '';
      for I := 0 to FColumnCount - 1 do
        if FRecordColumns[I].Exportable then
          lString := lString + MaybeQuote(FRecordColumns[I].ColumnName) + Separator;
      FDocument.Add(lString);

      while not Eof do
      begin
        lString := '';
        for I := 0 to FColumnCount - 1 do
          if FRecordColumns[I].Exportable then
          try
            if not FRecordColumns[I].Field.IsNull then
            begin
              lField := FRecordColumns[I].Field.{Kambi*AsString}Text;
              lString := lString + MaybeQuote(lField);
            end;
            lString := lString + Separator;
          except
            Result := False;
            raise;
          end;
        FDocument.Add(lString);
        Next;
        Inc(ARecNo);
        if not DoProgress(0, lRecCount, ARecNo, Caption) then
          Last;
      end;
      DoProgress(0, lRecCount, lRecCount, Caption);
    finally
      if BookmarkValid(lBookmark) then
        GotoBookmark(lBookmark);
      if lBookmark <> nil then
        FreeBookmark(lBookmark);
      EnableControls;
    end;
  end;
end;

procedure TKamDBGridCSVExport.DoSave;
begin
  inherited DoSave;
  if Destination = edFile then
    FDocument.SaveToFile(FileName)
  else
    Clipboard.AsText := FDocument.Text;
end;

procedure TKamDBGridCSVExport.DoClose;
begin
  // do nothing
end;

//=== { TKamDBGridXMLExport } =================================================

constructor TKamDBGridXMLExport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FXML := TJvSimpleXML.Create(nil);
  FXML.Options := [sxoAutoCreate, sxoAutoIndent];
  FEncoding := 'iso-8859-1';
end;

destructor TKamDBGridXMLExport.Destroy;
begin
  FXML.Free;
  inherited Destroy;
end;

// From DSDEfine of Delphi designer

function TKamDBGridXMLExport.ClassNameNoT(AField: TField): string;
begin
  Result := AField.ClassName;
  if Result[1] = 'T' then
    Delete(Result, 1, 1);
  if SameText('Field', Copy(Result, Length(Result) - 4, 5)) then { do not localize }
    Delete(Result, Length(Result) - 4, 5);
end;

// The structure of the xml file is inspired of the xml export
// create by Delphi with TClientDataSet

function TKamDBGridXMLExport.DoExport: Boolean;

  { This tries to return a unique name for a column that is a valid
    XML attribute name.

    (Simple Column.ColumnName cannot be used,
    because it's taken from Title.Caption of DBGrid column, so it's
    something that is meant to be displayed, shown to user, has spaces,
    local characters etc.) }
  function ColumnXMLName(const Column: TRecordColumn): string;
  begin
   { Try to use FieldName.
     If we can't, use ColumnName, but fixed (with special chars converted to '_'. } 
   if (Column.Field <> nil) and (Column.Field.FieldName <> '') then
    Result := Column.Field.FieldName else
    Result := SReplaceChars(Column.ColumnName,
      AllChars - ['a'..'z', 'A'..'Z', '0'..'9', '_'], '_');
  end;

var
  I: Integer;
  ARecNo, lRecCount: Integer;
  lBookmark: TBookmark;
  lRootNode: TJvSimpleXmlElemClassic;
  lDataNode: TJvSimpleXmlElem;
  lFieldsNode: TJvSimpleXmlElem;
  lRecordNode: TJvSimpleXmlElem;
begin
  Result := True;

  FXML.Prolog.Encoding := Encoding; 

  FXML.Root.Clear;

  // create root node
  FXML.Root.Name := 'DATAPACKET';
  lRootNode := FXML.Root;
  lRootNode.Properties.Add('Version', '1.0.Kambi');

  // add column header and his property
  lDataNode := lRootNode.Items.Add('METADATA');
  lFieldsNode := lDataNode.Items.Add('FIELDS');
  for I := 0 to FColumnCount - 1 do
    with FRecordColumns[I] do
      if Visible and (Field <> nil) then
      begin
        with lFieldsNode.Items.Add('FIELD') do
        begin
          Properties.Add('ATTRNAME', ColumnXMLName(FRecordColumns[I]));
          Properties.Add('FIELDTYPE', ClassNameNoT(Field));
          Properties.Add('WIDTH', Column.Width);
          Properties.Add('CAPTION', ColumnName);
        end;
      end;

  // now add all the record
  lRecordNode := lRootNode.Items.Add('ROWDATA');

  with Grid.DataSource.DataSet do
  begin
    ARecNo := 0;
    lRecCount := RecordCount;
    DoProgress(0, lRecCount, ARecNo, Caption);
    DisableControls;
    lBookmark := GetBookmark;
    First;
    try
      while not Eof do
      begin
        with lRecordNode.Items.Add('ROW') do
        begin
          for I := 0 to FColumnCount - 1 do
            if FRecordColumns[I].Exportable then
            try
              Properties.Add(ColumnXMLName(FRecordColumns[I]),
                FRecordColumns[I].Field.{Kambi*AsString}Text);
            except
              Result := False;
              raise;
            end;
        end;

        Next;
        Inc(ARecNo);
        if not DoProgress(0, lRecCount, ARecNo, Caption) then
          Last;
      end;
      DoProgress(0, lRecCount, lRecCount, Caption);
    finally
      if BookmarkValid(lBookmark) then
        GotoBookmark(lBookmark);
      if lBookmark <> nil then
        FreeBookmark(lBookmark);
      EnableControls;
    end;
  end;
end;

procedure TKamDBGridXMLExport.DoSave;
begin
  inherited DoSave;
  FXML.SaveToFile(FileName);
end;

procedure TKamDBGridXMLExport.DoClose;
begin
  // do nothing
end;

//============================================================================

type
  TGridValue = packed record
    Value: Integer;
    Name: PChar;
  end;

const
  GridFormats: array [$10..$17] of TGridValue =
   ((Value: $10; Name: 'wdTableFormatGrid1'),
    (Value: $11; Name: 'wdTableFormatGrid2'),
    (Value: $12; Name: 'wdTableFormatGrid3'),
    (Value: $13; Name: 'wdTableFormatGrid4'),
    (Value: $14; Name: 'wdTableFormatGrid5'),
    (Value: $15; Name: 'wdTableFormatGrid6'),
    (Value: $16; Name: 'wdTableFormatGrid7'),
    (Value: $17; Name: 'wdTableFormatGrid8'));

function WordGridFormatIdentToInt(const Ident: string; var Value: Longint): Boolean;
var
  I: Integer;
begin
  for I := Low(GridFormats) to High(GridFormats) do
    if SameText(GridFormats[I].Name, Ident) then
    begin
      Result := True;
      Value := GridFormats[I].Value;
      Exit;
    end;
  Result := False;
end;

function IntToWordGridFormatIdent(Value: Longint; var Ident: string): Boolean;
var
  I: Integer;
begin
  for I := Low(GridFormats) to High(GridFormats) do
    if GridFormats[I].Value = Value then
    begin
      Result := True;
      Ident := GridFormats[I].Name;
      Exit;
    end;
  Result := False;
end;

procedure GetWordGridFormatValues(Proc: TGetStrProc);
var
  I: Integer;
begin
  for I := Low(GridFormats) to High(GridFormats) do
    Proc(GridFormats[I].Name);
end;

{$IFDEF UNITVERSIONING}
const
  UnitVersioning: TUnitVersionInfo = (
    RCSfile: '$RCSfile: JvDBGridExport.pas,v $';
    Revision: '$Revision: 1.26 $';
    Date: '$Date: 2004/09/01 15:06:35 $';
    LogPath: 'JVCL\run'
  );
{$ENDIF UNITVERSIONING}

initialization
  {$IFDEF UNITVERSIONING}
  RegisterUnitVersion(HInstance, UnitVersioning);
  {$ENDIF UNITVERSIONING}

  RegisterIntegerConsts(TypeInfo(TWordGridFormat), WordGridFormatIdentToInt, IntToWordGridFormatIdent);


{$IFDEF UNITVERSIONING}
finalization
  UnregisterUnitVersion(HInstance);
{$ENDIF UNITVERSIONING}

end.

